AJS.$(document).ready(function() {
	var timeline=new vis.Timeline(AJS.$("#timeline")[0]);
	var timelineData={
		options: null,
		groups: null,
		items: null
	};
	
	timeline.on('select', function (properties) {
		AJS.$("[instanceId]").removeClass("vis-selected");
		for (var i=0; i<properties.items.length; i++) {
			var id=properties.items[i];
			if (typeof id === 'string' && id.includes("used-"))
				id=id.split("-")[1];
			AJS.$("[instanceId="+id+"]").addClass("vis-selected");
		}
	});
	
	AJS.$("#data").hide();
	
	AJS.$(".userSelect").auiSelect2({placeholder: AJS.I18n.getText("vacationmanager.action.statistics.user.select.user.placeholder")});
	AJS.$("#startDate").datePicker({'overrideBrowserDefault': true});
	AJS.$("#endDate").datePicker({'overrideBrowserDefault': true});
	
	AJS.$("#startDate").val(parseDate(new Date()));
	AJS.$("#endDate").val(parseDate(new Date(new Date().getFullYear(), 11, 31)));
	
	function changeTableData(user, start, end) {
		AJS.$("#loading-overlay").show();
		var params={userKey:user, startDate:start, endDate:end};
		jQuery.ajax({
			type : "POST",
			url : AJS.contextPath() + "/rest/vacation-manager/1.0/hrmanagement/statistics/instances/user",
			data : JSON.stringify(params),
			dataType : "json",
			contentType: 'application/json; charset=utf-8',
			async : true
		}).done(function(data) {
			AJS.$("#loading-overlay").hide();
			populateData(data);
		});
	}
	
	function populateData(data) {
		populateInfoHeader(data);
		clearTimeline();
		
		AJS.$("[vacationKind]").each(function() {
			var obj=AJS.$(this);
			var chart=obj.find("[name=chart]");
			var table=obj.find("[name=table]");
			var vacationKindId=obj.attr("vacationKind");
			var userData=data.vacationKindInstances[vacationKindId];

			populateTableData(userData, table);
			populateChartData(userData, chart);
			populateTimelineData(userData, data.freeDays);
		});
		renderTimeline(new Date(data.startDate), new Date(data.endDate));
	}
	
	function populateInfoHeader(data) {
		var label = AJS.I18n.getText("vacationmanager.action.statistics.user.displaying.for.label", data.user.name, data.user.displayName, parseDate(new Date(data.startDate)), parseDate(new Date(data.endDate)));
		AJS.$("#dataInfo").text(label);
	}
	
	function clearTimeline() {
		timelineData.items = new vis.DataSet();
		timelineData.groups = new vis.DataSet();
	}
	
	function populateTimelineData(userData, freeDays) {
		if (userData!=undefined) {
			var vacationKindId=userData.vacationKind.id;
			var vacationKindName=AJS.$("<p/>", {text: userData.vacationKind.name}).html();
			timelineData.groups.add({id: vacationKindId, content: vacationKindName});
			
			if (userData.instances.length>0) {
				for (var inst=0; inst<userData.instances.length; inst++) {
					var instanceFromRest=userData.instances[inst];
					var documents=instanceFromRest.vacationDocuments;
					
				    for (var doc=0; doc<documents.length; doc++) {
				    	var document=documents[doc];
				    	var itemContent=AJS.I18n.getText("vacationmanager.action.statistics.user.timeline.item.used", document.calculatedDays);
				    	timelineData.items.add({id:"used-"+instanceFromRest.id+"-"+document.id, group: vacationKindId, content: itemContent,
				    		start: new Date(document.startDate), end: new Date(document.endDate).endOfDay(), className: "vacman-vis-used"});
				    }
				}
			}

			if (freeDays!=undefined) {
				for (var i=0; i<freeDays.length; i++) {
					var day = freeDays[i];
			    	timelineData.items.add({group: vacationKindId, start: new Date(day), end: new Date(day).endOfDay(), className: "vacman-vis-free", type:"background"});
				}
			}
		
		}
	}
	
	function renderTimeline(startDate, endDate) {
		timelineData.options = { min: startDate, max: endDate.endOfDay(), start: startDate, end: endDate.endOfDay(),
			margin:{item:{horizontal:0}}, timeAxis:{scale:"day"}, type:"range", zoomMin:172800000 };
		timeline.setOptions(timelineData.options);
		timeline.setGroups(timelineData.groups);
		timeline.setItems(timelineData.items);
	}
	
	function populateTableData(userData, table) {
        var notApplicableLabel=AJS.I18n.getText("vacationmanager.text.not.applicable");
	
		var now = new Date();
		
		if (userData!=undefined) {
			if (userData.instances.length>0) {
				var instances = new Array();
				var availableDaysSum=0;
				var carryOverDaysSum=0;
				var usedDaysSum=0;
				for (var inst=0; inst<userData.instances.length; inst++) {
					var instanceFromRest=userData.instances[inst];
					var current = true;
					var documents=instanceFromRest.vacationDocuments;
					var startDate=new Date(instanceFromRest.startDate);
					var endDate=new Date(instanceFromRest.endDate);
					var availableDays=instanceFromRest.availableDays;
					var carryOverDays=instanceFromRest.carryOverDays;
					var usedDays=instanceFromRest.usedDays;

					if (startDate>now) current = false; //future
					else if (endDate<now) current = false; //past
					
					var instance={
						id:instanceFromRest.id,
						startDate:parseDate(startDate),
						endDate:(userData.vacationKind.validThrough==false)?notApplicableLabel:parseDate(endDate),
						availableDays:(userData.vacationKind.hasDayLimit==false)?notApplicableLabel:availableDays,
						carryOverDays:carryOverDays,
						usedDays:usedDays,
						vacationDocuments:documents,
						current: current
					};

					availableDaysSum+=parseInt(instanceFromRest.availableDays);
					carryOverDaysSum+=parseInt(instanceFromRest.carryOverDays);
					usedDaysSum+=parseInt(instanceFromRest.usedDays);
					instances.push(instance);
				}
				var generatedTable=VacationManager.Templates.UserStatistics.vacationKindInstancesTable({instances:instances, jqlSearchUrl:AJS.params.baseURL+"/issues/?jql="});
				table.html(generatedTable);
			} else {
				table.html(AJS.I18n.getText("vacationmanager.action.statistics.no.definitions"));
			}
		} else {
			table.html(AJS.I18n.getText("vacationmanager.action.statistics.no.data"));
		}
	}
	
	function populateChartData(userData, chart) {
		if (userData!=undefined) {
			if (userData.instances.length>0) {
				var instances = new Array();
				var availableDaysSum=0;
				var carryOverDaysSum=0;
				var usedDaysSum=0;
				
				for (var inst=0; inst<userData.instances.length; inst++) {
					var instanceFromRest=userData.instances[inst];
					var documents=instanceFromRest.vacationDocuments;
					
					var startDate=new Date(instanceFromRest.startDate);
					var endDate=new Date(instanceFromRest.endDate);
					var availableDays=instanceFromRest.availableDays;
					var carryOverDays=instanceFromRest.carryOverDays;
					var usedDays=instanceFromRest.usedDays;

					availableDaysSum+=parseInt(instanceFromRest.availableDays);
					carryOverDaysSum+=parseInt(instanceFromRest.carryOverDays);
					usedDaysSum+=parseInt(instanceFromRest.usedDays);
				}
				drawChart(chart[0], availableDaysSum, carryOverDaysSum, usedDaysSum);
			} else {
				chart.html("");
			}
		} else {
			chart.html("");
		}
	}
	
	var yearMilis = 1000*60*60*24*365;
	
	function checkChangedData() {
		AJS.$("#error").text("");
	
		var user=AJS.$("#userSelect").val();
		var startDate=new Date(AJS.$("#startDate").val());
		var endDate=new Date(AJS.$("#endDate").val());
		
		if (startDate=="Invalid Date") {
			AJS.$("#error").text(AJS.I18n.getText("vacationmanager.vacation.validator.invalid.start.date"));
			return;
		}
		
		if (endDate=="Invalid Date") {
			AJS.$("#error").text(AJS.I18n.getText("vacationmanager.vacation.validator.invalid.end.date"));
			return;
		}
		
		if (startDate>endDate) {
			AJS.$("#error").text(AJS.I18n.getText("vacationmanager.vacation.validator.enddate.before.startdate"));
			return;
		}
		
		if (endDate-startDate > yearMilis * 10) {
			AJS.$("#error").text(AJS.I18n.getText("vacationmanager.vacation.validator.period.too.long"));
			return;
		}
		
		if (user.length<=0) {
			return;
		}
		
		AJS.$("#data").show();
		changeTableData(user, startDate, endDate);
	}

  	function drawChart(div, availableDays, leftoverDays, usedDays) {
  		if (availableDays<0) availableDays=0;
  		if (leftoverDays<0) leftoverDays=0;
  		if (usedDays<0) usedDays=0;
  		
  		google.charts.setOnLoadCallback(function() {
	        var data = new google.visualization.DataTable();
			data.addColumn('string', 'Days');
			data.addColumn('number', 'Amount');
			data.addColumn({
				type : 'string',
				role : 'tooltip'
			});

			var availableDaysLabel = AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.available.days.description", availableDays);
			var leftoverDaysLabel = (leftoverDays>0)?" "+AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.leftover.days.description", leftoverDays):"";

	        data.addRows([
				[ AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.available.days"), availableDays, availableDaysLabel + leftoverDaysLabel ],
				[ AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.used.days"), usedDays,
						AJS.I18n.getText("vacationmanager.vacation.customfield.statistics.used.days.description", usedDays) ]
			]);
		
			var options = {
				chartArea:{left:10,top:10,width:'80%',height:180},
				legend: {textStyle: {color: '#707070'}},
				colors : [
				           '#67ab49', //available/entitled days
				           '#d04437' //used days
			           ],
				pieSliceText : 'value'
			};
	
	        var chart = new google.visualization.PieChart(div);
	        chart.draw(data, options);
        });
  	}
  	
	AJS.$("[onChangeListened]").change(checkChangedData);
	if (AJS.$("[onInitListened]").length>0) {
		checkChangedData();
	};
	AJS.$('.overlay-spinner').spin();

	Date.prototype.endOfDay = function()
	{
	    var dat = new Date(this.valueOf());
	    dat.setDate(dat.getDate() + 1);
	    return dat;
	}
	
	function parseDate(date) {
		var year=date.getFullYear();
		var month=pad((date.getMonth()+1),2);
		var day=pad((date.getDate()),2);
		return year+"-"+month+"-"+day;
	}
	
	function pad(num, size) {
	    var s = "00000000000000" + num;
	    return s.substr(s.length-size);
	}
	
	window.onbeforeunload=null;
});
