function TableFilter(_addButton, _filterLabelsContainer, _table) {
	
	var counter=10000;
	var addButton=AJS.$("#"+_addButton);
	var addFilterMenu=null;
	var filterContainer=AJS.$("#"+_filterLabelsContainer);
	var table=_table;
	var updateCallback=null;
	
	function init() {
		var addButtonId=addButton.attr('aria-controls');
		addFilterMenu=AJS.$("#"+addButtonId).children();
		VM$.fn.dataTable.ext.search.push(filterFunction);
	}
	
	function addFilterDefinition(name, column, type, unique) {
		var item=AJS.$("<li><a href='#'>"+name+"</a></li>");
		item.attr("name", name);
		item.filterDefinition={name:name,
				column:column,
				type:type,
				unique:unique
		};
		addFilterMenu.append(item);
		item.click(function() {
			addFilterElement(item);
		});
	}
	
	function addFilterElement(item) {
		var def=item.filterDefinition;
		if (def.unique==true && filterContainer.find("[name="+def.name+"][column="+def.column+"][type="+def.type+"]").length>0) {
			return false;
		}
		var id="filter_element_"+(counter++);
		var label=VacationManager.Templates.Filter.filterLabel({id:id, name:def.name, column:def.column, type:def.type});
		var filter=null;
		if (def.type=="num")
			filter=VacationManager.Templates.Filter.filterNum({id:id, name:def.name, column:def.column, type:def.type});
		else
			filter=VacationManager.Templates.Filter.filterText({id:id, name:def.name, column:def.column, type:def.type});
		filterContainer.append(label);
		
		var inlineDialog = AJS.InlineDialog(AJS.$("#label-"+id), "dialog-"+id,
                function (content, trigger, showPopup) {
                    content.html(filter);
                    content.click(function(evt) {
                    	if (AJS.$(evt.target).attr("name")!="filter-ok") {
                    		evt.stopPropagation();
                    	}
                    });
                    showPopup();
                    return false;
                },
                {
                	"cacheContent": true,
                	"hideCallback": function() {
        				updateButtonLabel(id, def.type);
        				applyFilters();
                	},
                	"initCallback": function() {
                		setTimeout(function() {inlineDialog.find("input[name=value]").focus();}, 150);
                	},
                	"hideDelay": null,
                	"width": 350
                }
        );
		inlineDialog.show();

		filterContainer.find("#label-"+id).find(".remove-filter").click(function() {
			removeFilterElement(id);
			applyFilters();
		});


		return true;
	}
	
	function updateButtonLabel(id, type) {
		var text="";
		var obj=AJS.$("#"+id);
		if (type=="num") {
			var num=parseInt(obj.find("input[name=value]").val());
			if (!isNaN(num)) {
				var action=obj.find("select[name=action]").val();
				switch (action) {
					case "less":
						text="<";
					break;
					
					case "equal":
						text="=";
					break;
					
					case "more":
						text=">";
					break;
				}
				text+=" "+num;
			}
		} else {
			var val=obj.find("input[name=value]").val();
			if (val != null && val.length>0)
				text="'"+val+"'";
		}
		AJS.$("span[name="+id+"-label]").text(text);
	}
	
	function removeFilterElement(id) {
		AJS.$("#label-"+id).remove();
		AJS.$("#"+id).remove();
	}

	function applyFilters() {
		if (updateCallback!=null)
			updateCallback();
	}
	
	function filterFunction( settings, data, dataIndex ) {
		var filters=getFilters();
		return checkFilters(data, filters);
	}
	
	function checkFilters(row, filters) {
		for (var i=0; i<filters.length; i++) {
			var filter=filters[i];
			if (filter.type=="num") {
				if (checkNumericFilter(row, filter.column, filter.value, filter.action)==false)
					return false;
			} else {
				if (checkTextFilter(row, filter.column, filter.value)==false)
					return false;
			}
		}
		return true;
	}

	function getFilters() {
		var filters=new Array();
		AJS.$("[name='filter-element']").each(function() {
			var elem=AJS.$(this);
			var obj={};
			obj.column=elem.attr("column");
			obj.type=elem.attr("type");
			if (obj.type=="num") {
				obj.action=elem.find("[name=action]").val();
				obj.value=parseInt(elem.find("[name=value]").val());
				if (!isNaN(obj.value)) {
					filters.push(obj);
				} 
			} else {
				obj.value=elem.find("[name=value]").val();
				if (obj.value.length>0)
					filters.push(obj);
			}
		});
		return filters;
	}
	
	function getFiltersString() {
		var query = "";
		AJS.$("[name='filter-element']").each(function() {
			query += AJS.$("#label-"+this.id).text() + " AND ";
		});
		query = query.substring(0, query.length-4);
		return query;
	}
	
	function checkTextFilter(rowData, checkColumn, text) {
		if (rowData[checkColumn].toLowerCase().includes(text)==false)
			return false;
		return true;
	}
	
	function checkNumericFilter(rowData, checkColumn, number, sign) {
		var checkNumber=parseInt(rowData[checkColumn]);
		number=parseInt(number);
		if (number==undefined || checkNumber==undefined)
			return false;
		switch(sign) {
			case "less":
				if (checkNumber<number)
					return true;
			break;
			
			case "equal":
				if (checkNumber==number)
					return true;
			break;
			
			case "more":
				if (checkNumber>number)
					return true;
			break;
		}
		return false;
	}
	
	function setUpdateCallback(callback) {
		updateCallback=callback;
	}

	init();
	
	return {
		addFilterDefinition:addFilterDefinition,
		getFilters:getFilters,
		getFiltersString: getFiltersString,
		setUpdateCallback:setUpdateCallback
	}
}
