var HolidayDialogs = function() {
    
    var editEvent = null;
    var removeEvent = null;
    
    function initAddHolidayDialog(allHolidaySchemes, selectedHolidaySchemeId, addedCallback) {
        AJS.$("body").append(VacationManager.Templates.Workdays.Holidays.HolidayDialogs
                .addHolidayDialog({allHolidaySchemes: allHolidaySchemes, currentHolidaySchemeId: holidaySchemeId}));
        
        AJS.$("#addHolidayDialogName").keypress(function(event){
            if(event.keyCode == 13){
                event.preventDefault();
                $("#addHolidayDialogSubmit").click();
                return false;
            }
        });

        AJS.$("#addHolidayDialogSubmit").click(function(e) {
            e.preventDefault();
            AJS.dialog2("#addHolidayDialog").hide();
            
            var day = VM$.moment(AJS.$("#addHolidayDialogDay").val());
            var name = AJS.$("#addHolidayDialogName").val();
            var yearly = ($('input[name=yearly]:checked').val()=="true");
            var holidaySchemes = AJS.$("#holidaySchemeIds").val();
            addHoliday(day, yearly, name, holidaySchemes);
        });
        
        AJS.$("#addHolidayDialogClose").click(function(e) {
            e.preventDefault();
            AJS.dialog2("#addHolidayDialog").hide();
        });

        function addHoliday(date, yearly, name, holidaySchemes) {
            if (holidaySchemes == null) {
                return;
            }

            var params = [];
            
            for (var scheme=0; scheme<holidaySchemes.length; scheme++) {
                params.push({date: date.format('YYYY-MM-DD'), yearly: yearly, description: name,
                        holidaySchemeId: holidaySchemes[scheme]});
            }
        
            jQuery.ajax({
                type: "PUT",
                url: AJS.contextPath() + "/rest/vacation-manager/1.0/holiday-management/holidays",
                data : JSON.stringify(params),
                dataType : "json",
                contentType: "application/json; charset=UTF-8",
                success: function(obj) {
                    JIRA.Messages.showSuccessMsg(
                            AJS.I18n.getText("vacationmanager.action.holiday.holidays.add.msg.succesfull", name, obj.length), {
                                closeable : true
                            });
                    if (addedCallback != undefined) {
                        addedCallback(obj);
                    }
                },
                error: function(obj) {
                    var errors = VM$.parseJSON(obj.responseText).errors;
                    for (var i=0; i<errors.length; i++) {
                        JIRA.Messages.showErrorMsg(errors[i].message, { closeable : true })
                    }
                }
            });
        }
        
        AJS.$('.aui-date-picker').each(function() {
            AJS.$(this).datePicker({'overrideBrowserDefault': true});
        });
        AJS.$("#holidaySchemeIds").auiSelect2({placeholder: AJS.I18n.getText("vacationmanager.action.holiday.holidays.select.holiday.schemes")});
        
        AJS.$("[holiday-list-add-new]").click(function(evt) {
            showAddHolidayDialog();
        });
    }
    
    function initEditHolidayDialog(editedCallback) {
        AJS.$("body").append(VacationManager.Templates.Workdays.Holidays.HolidayDialogs.editHolidayDialog());

        AJS.$("#editHolidayDialogName").keypress(function(event){
            if(event.keyCode == 13){
                event.preventDefault();
                $("#editHolidayDialogSubmit").click();
                return false;
            }
        });
        
        AJS.$("#editHolidayDialogSubmit").click(function(e) {
            e.preventDefault();
            AJS.dialog2("#editHolidayDialog").hide();

            editEvent.title = AJS.$("#editHolidayDialogName").val();
            editEvent.yearly = (AJS.$('input[name=editYearly]:checked').val()=="true");
            editHoliday(editEvent);
        });
        
        AJS.$("#editHolidayDialogClose").click(function(e) {
            e.preventDefault();
            AJS.dialog2("#editHolidayDialog").hide();
        });

        function editHoliday(editEvent) {
            var params = {id: editEvent.id, date: editEvent.start.format('YYYY-MM-DD'), yearly: editEvent.yearly, description: editEvent.title,
                    holidaySchemeId: holidaySchemeId};
            
            jQuery.ajax({
                type: "PUT",
                url: AJS.contextPath() + "/rest/vacation-manager/1.0/holiday-management/holiday",
                data : JSON.stringify(params),
                dataType : "json",
                contentType: "application/json; charset=UTF-8",
                success: function(obj) {    
                    if (editedCallback != undefined) {
                        editedCallback(obj);
                    }
                }
            });
        }
    }
    
    function initRemoveHolidayDialog(removedCallback) {
        AJS.$("body").append(VacationManager.Templates.Workdays.Holidays.HolidayDialogs.removeHolidayDialog());
        
        AJS.$("#removeHolidayDialogOpen").click(function(e) {
            e.preventDefault();
            removeEvent = editEvent;
            showRemoveHolidayDialog(removeEvent);
        });
        
        AJS.$("#removeHolidayDialogSubmit").click(function(e) {
            e.preventDefault();
            AJS.dialog2("#removeHolidayDialog").hide();

            removeHoliday(removeEvent);
        });
        
        AJS.$("#removeHolidayDialogClose").click(function(e) {
            e.preventDefault();
            AJS.dialog2("#removeHolidayDialog").hide();
        });

        function removeHoliday(removeEvent) {
            jQuery.ajax({
                type: "DELETE",
                url: AJS.contextPath() + "/rest/vacation-manager/1.0/holiday-management/holiday?id="+removeEvent.id,
                contentType: "application/json; charset=UTF-8",
                success: function() {   
                    if (removedCallback != undefined) {
                        removedCallback(removeEvent.id);
                    }
                }
            });
        }
    }
    
    function showAddHolidayDialog(date) {
        if (date == null) {
            date = VM$.moment();
        }
        AJS.$("#addHolidayDialogName").val("");
        var day=date.format('YYYY-MM-DD');
        AJS.$("#addHolidayDialogDay").val(day);
        AJS.$("#addHolidayDialogDay").trigger("input");
        $("input[name=yearly][value='false']").prop("checked",true);
        AJS.dialog2("#addHolidayDialog").show();
        AJS.$("#addHolidayDialogName").focus();
    }
    
    function showEditHolidayDialog(calendarEvent) {
        editEvent = calendarEvent;
        AJS.$("#editHolidayDialogName").val(editEvent.title);
        var day=editEvent.start.format('YYYY-MM-DD');
        AJS.$("#editHolidayDialogDay").text(day);
        if (editEvent.yearly) {
            AJS.$("input[name=editYearly][value='true']").prop("checked",true);
        } else {
            AJS.$("input[name=editYearly][value='false']").prop("checked",true);
        }
        AJS.dialog2("#editHolidayDialog").show();
    }
    
    function showRemoveHolidayDialog(calendarEvent) {
        removeEvent = calendarEvent;
        var day=removeEvent.start.format('YYYY-MM-DD');
        var name=removeEvent.title;
        var yearly=removeEvent.yearly;
        if (yearly) {
            AJS.$("#removeHolidayOnetime").hide();
            AJS.$("#removeHolidayYearly").show();
        } else {
            AJS.$("#removeHolidayOnetime").show();
            AJS.$("#removeHolidayYearly").hide();
        }
        AJS.$("#removeHolidayDialogDay").text(day);
        AJS.$("#removeHolidayDialogName").text(name);
        
        AJS.dialog2("#editHolidayDialog").hide();
        AJS.dialog2("#removeHolidayDialog").show();
    }

    return {        
        initAddHolidayDialog:initAddHolidayDialog,
        initEditHolidayDialog:initEditHolidayDialog,
        initRemoveHolidayDialog:initRemoveHolidayDialog,
        
        showAddHolidayDialog:showAddHolidayDialog,
        showEditHolidayDialog:showEditHolidayDialog,
        showRemoveHolidayDialog:showRemoveHolidayDialog
    } 
}();
